/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.reactivex.mssqlclient;

import io.vertx.reactivex.RxHelper;
import io.vertx.reactivex.ObservableHelper;
import io.vertx.reactivex.FlowableHelper;
import io.vertx.reactivex.impl.AsyncResultMaybe;
import io.vertx.reactivex.impl.AsyncResultSingle;
import io.vertx.reactivex.impl.AsyncResultCompletable;
import io.vertx.reactivex.WriteStreamObserver;
import io.vertx.reactivex.WriteStreamSubscriber;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * A  of {@link io.vertx.reactivex.mssqlclient.MSSQLConnection SQL Server connections}.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.mssqlclient.MSSQLPool original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.mssqlclient.MSSQLPool.class)
public class MSSQLPool extends io.vertx.reactivex.sqlclient.Pool {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    MSSQLPool that = (MSSQLPool) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<MSSQLPool> __TYPE_ARG = new TypeArg<>(    obj -> new MSSQLPool((io.vertx.mssqlclient.MSSQLPool) obj),
    MSSQLPool::getDelegate
  );

  private final io.vertx.mssqlclient.MSSQLPool delegate;
  
  public MSSQLPool(io.vertx.mssqlclient.MSSQLPool delegate) {
    super(delegate);
    this.delegate = delegate;
  }

  public MSSQLPool(Object delegate) {
    super((io.vertx.mssqlclient.MSSQLPool)delegate);
    this.delegate = (io.vertx.mssqlclient.MSSQLPool)delegate;
  }

  public io.vertx.mssqlclient.MSSQLPool getDelegate() {
    return delegate;
  }

  /**
   * Like {@link io.vertx.reactivex.mssqlclient.MSSQLPool#pool} with a default <code>poolOptions</code>.
   * @param connectionUri 
   * @return 
   */
  public static io.vertx.reactivex.mssqlclient.MSSQLPool pool(java.lang.String connectionUri) { 
    io.vertx.reactivex.mssqlclient.MSSQLPool ret = io.vertx.reactivex.mssqlclient.MSSQLPool.newInstance((io.vertx.mssqlclient.MSSQLPool)io.vertx.mssqlclient.MSSQLPool.pool(connectionUri));
    return ret;
  }

  /**
   * Like {@link io.vertx.reactivex.mssqlclient.MSSQLPool#pool} with <code>connectOptions</code> built from <code>connectionUri</code>.
   * @param connectionUri 
   * @param poolOptions 
   * @return 
   */
  public static io.vertx.reactivex.mssqlclient.MSSQLPool pool(java.lang.String connectionUri, io.vertx.sqlclient.PoolOptions poolOptions) { 
    io.vertx.reactivex.mssqlclient.MSSQLPool ret = io.vertx.reactivex.mssqlclient.MSSQLPool.newInstance((io.vertx.mssqlclient.MSSQLPool)io.vertx.mssqlclient.MSSQLPool.pool(connectionUri, poolOptions));
    return ret;
  }

  /**
   * Like {@link io.vertx.reactivex.mssqlclient.MSSQLPool#pool} with a default <code>poolOptions</code>..
   * @param vertx 
   * @param connectionUri 
   * @return 
   */
  public static io.vertx.reactivex.mssqlclient.MSSQLPool pool(io.vertx.reactivex.core.Vertx vertx, java.lang.String connectionUri) { 
    io.vertx.reactivex.mssqlclient.MSSQLPool ret = io.vertx.reactivex.mssqlclient.MSSQLPool.newInstance((io.vertx.mssqlclient.MSSQLPool)io.vertx.mssqlclient.MSSQLPool.pool(vertx.getDelegate(), connectionUri));
    return ret;
  }

  /**
   * Like {@link io.vertx.reactivex.mssqlclient.MSSQLPool#pool} with <code>connectOptions</code> built from <code>connectionUri</code>.
   * @param vertx 
   * @param connectionUri 
   * @param poolOptions 
   * @return 
   */
  public static io.vertx.reactivex.mssqlclient.MSSQLPool pool(io.vertx.reactivex.core.Vertx vertx, java.lang.String connectionUri, io.vertx.sqlclient.PoolOptions poolOptions) { 
    io.vertx.reactivex.mssqlclient.MSSQLPool ret = io.vertx.reactivex.mssqlclient.MSSQLPool.newInstance((io.vertx.mssqlclient.MSSQLPool)io.vertx.mssqlclient.MSSQLPool.pool(vertx.getDelegate(), connectionUri, poolOptions));
    return ret;
  }

  /**
   * Create a connection pool to the SQL server configured with the given <code>connectOptions</code> and <code>poolOptions</code>.
   * @param connectOptions the options for the connection
   * @param poolOptions the options for creating the pool
   * @return the connection pool
   */
  public static io.vertx.reactivex.mssqlclient.MSSQLPool pool(io.vertx.mssqlclient.MSSQLConnectOptions connectOptions, io.vertx.sqlclient.PoolOptions poolOptions) { 
    io.vertx.reactivex.mssqlclient.MSSQLPool ret = io.vertx.reactivex.mssqlclient.MSSQLPool.newInstance((io.vertx.mssqlclient.MSSQLPool)io.vertx.mssqlclient.MSSQLPool.pool(connectOptions, poolOptions));
    return ret;
  }

  /**
   * Like {@link io.vertx.reactivex.mssqlclient.MSSQLPool#pool} with a specific  instance.
   * @param vertx 
   * @param connectOptions 
   * @param poolOptions 
   * @return 
   */
  public static io.vertx.reactivex.mssqlclient.MSSQLPool pool(io.vertx.reactivex.core.Vertx vertx, io.vertx.mssqlclient.MSSQLConnectOptions connectOptions, io.vertx.sqlclient.PoolOptions poolOptions) { 
    io.vertx.reactivex.mssqlclient.MSSQLPool ret = io.vertx.reactivex.mssqlclient.MSSQLPool.newInstance((io.vertx.mssqlclient.MSSQLPool)io.vertx.mssqlclient.MSSQLPool.pool(vertx.getDelegate(), connectOptions, poolOptions));
    return ret;
  }

  public static MSSQLPool newInstance(io.vertx.mssqlclient.MSSQLPool arg) {
    return arg != null ? new MSSQLPool(arg) : null;
  }

}
